@@ -4,7 +4,7 @@ package JSON::Schema::Draft201909::Vocabulary::Applicator;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema Draft 2019-09 Applicator vocabulary
 
-our $VERSION = '0.018';
+our $VERSION = '0.019';
 
 use 5.016;
 no if "$]" >= 5.031009, feature => 'indirect';
@@ -373,19 +373,15 @@ sub _eval_keyword_contains {
     return 0 if $state->{short_circuit};
   }
 
-  if (exists $schema->{maxContains}) {
-    local $state->{keyword} = 'maxContains';
-
-    if ($num_valid > $schema->{maxContains}) {
-      $valid = 0;
-      E($state, 'contains too many matching items');
-      return 0 if $state->{short_circuit};
-    }
+  # TODO: in the future, we can move these implementations to the Validation vocabulary
+  # and inspect the annotation produced by the 'contains' keyword.
+  if (exists $schema->{maxContains} and $num_valid > $schema->{maxContains}) {
+    $valid = E({ %$state, keyword => 'maxContains' }, 'contains too many matching items');
+    return 0 if $state->{short_circuit};
   }
 
   if ($num_valid < ($schema->{minContains} // 1)) {
-    $valid = 0;
-    E({ %$state, keyword => 'minContains' }, 'contains too few matching items');
+    $valid = E({ %$state, keyword => 'minContains' }, 'contains too few matching items');
     return 0 if $state->{short_circuit};
   }
 
@@ -639,7 +635,7 @@ JSON::Schema::Draft201909::Vocabulary::Applicator - Implementation of the JSON S
 
 =head1 VERSION
 
-version 0.018
+version 0.019
 
 =head1 DESCRIPTION
 
